package company_contract

import (
	"github.com/beego/beego/v2/client/orm"
	"hongze/hongze_mobile_admin/models/tables/company_report_permission"
	"time"
)

type CompanyContract struct {
	CompanyContractId int       `orm:"column(company_contract_id);pk" description:"客户合同id"`
	ContractType      string    `description:"合同类型：枚举值：'新签合同','续约合同','补充协议'"`
	ProductId         int       `description:"产品id"`
	ProductName       string    `description:"产品名称"`
	CompanyId         int       `description:"客户id"`
	CompanyProductId  int       `description:"客户产品id"`
	ContractCode      string    `description:"合同编码"`
	StartDate         string    `description:"合同开始时间"`
	EndDate           string    `description:"合同结束时间"`
	Money             float64   `description:"合同金额"`
	PayMethod         string    `description:"支付方式"`
	PayChannel        string    `description:"支付渠道"`
	ImgUrl            string    `description:"合同图片，多个用#隔开"`
	CreateTime        time.Time `description:"合同创建时间"`
	ModifyTime        time.Time `description:"合同修改时间"`
	Status            int       `description:"状态"`
	PackageType       int       `description:"套餐类型，0：无，1：大套餐，2：小套餐"`
}

type CompanyContractDetail struct {
	CompanyContractId int
	CompanyId         int       `description:"客户id"`
	ProductId         int       `description:"产品id" json:"-"`
	ContractCode      string    `description:"合同编码"`
	ContractType      string    `description:"合同类型：枚举值：'新签合同','续约合同','补充协议'"`
	StartDate         string    `description:"合同开始时间"`
	EndDate           string    `description:"合同结束时间"`
	Money             float64   `description:"合同金额"`
	PayMethod         string    `description:"支付方式"`
	PayChannel        string    `description:"支付渠道"`
	ImgUrl            string    `description:"合同图片，多个用#隔开"`
	CreateTime        time.Time `description:"合同创建时间"`
	ModifyTime        time.Time `description:"合同修改时间"`
	Status            int       `description:"状态"`
	PackageType       int       `description:"套餐类型，0：无，1：大套餐，2：小套餐"`
	PermissionList    []*company_report_permission.PermissionLookList
}

func GetCompanyContractDetail(companyId, productId, companyContractId int) (item *CompanyContractDetail, err error) {
	o := orm.NewOrm()
	if companyContractId == 0 {
		sql := `SELECT * FROM company_contract WHERE company_id=? AND product_id=? AND status=0 `
		err = o.Raw(sql, companyId, productId).QueryRow(&item)
	} else {
		sql := `SELECT * FROM company_contract WHERE company_contract_id=? `
		err = o.Raw(sql, companyContractId).QueryRow(&item)
	}
	return
}

// GetCompanyContractPermissionCheckByContractId 根据合同id获取已经勾选的权限
func GetCompanyContractPermissionCheckByContractId(companyId, companyContractId, permissionId int) (count int, err error) {
	o := orm.NewOrm()
	sql := `SELECT COUNT(1) AS count FROM company_contract_permission AS a WHERE a.company_id=? AND company_contract_id=? AND a.chart_permission_id=? `
	err = o.Raw(sql, companyId, companyContractId, permissionId).QueryRow(&count)
	return
}
