package variety_tag

import (
	"github.com/beego/beego/v2/client/orm"
	"time"
)

// VarietyTag 标签表
type VarietyTag struct {
	VarietyTagId                int       `orm:"column(variety_tag_id);pk" description:"标签ID"`
	VarietyClassifyId           int       `json:"variety_classify_id" description:"标签分类ID"`
	VarietyClassifyName         string    `json:"variety_classify_name" description:"标签分类名称"`
	TagName                     string    `json:"tag_name" description:"标签名称"`
	ChartPermissionClassifyName string    `json:"chart_permission_classify_name" description:"品种权限分类名"`
	ChartPermissionId           int       `json:"chart_permission_id" description:"品种权限ID"`
	ChartPermissionName         string    `json:"chart_permission_name" description:"品种权限名称"`
	Sort                        int       `json:"sort" description:"排序"`
	State                       int       `json:"state" description:"显示状态 0-禁用 1-启用"`
	CreateTime                  time.Time `json:"create_time" description:"创建时间"`
	ModifyTime                  time.Time `json:"modify_time" description:"更新时间"`
}

// GetVarietyTagById 主键获取标签
func GetVarietyTagById(tagId int) (item *VarietyTag, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM variety_tag WHERE variety_tag_id = ? LIMIT 1`
	err = o.Raw(sql, tagId).QueryRow(&item)
	return
}

// GetVarietyTagList 获取标签列表
func GetVarietyTagList() (list []*VarietyTag, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM variety_tag WHERE state = 1 ORDER BY sort`
	_, err = o.Raw(sql).QueryRows(&list)
	return
}

// GetVarietyTagListByClassifyId 根据分类获取标签列表
func GetVarietyTagListByClassifyId(classifyId int) (list []*VarietyTag, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM variety_tag WHERE variety_classify_id = ? AND state = 1 ORDER BY sort`
	_, err = o.Raw(sql).QueryRows(&list)
	return
}
